function SLIP_stab_bound(varargin)

%% Copyright bel Mihly NAGY, Dra PATK, Ambrus ZELEI
%Discovery and Online Interactive Representation of the Dimensionless Parameter-Space of the Spring-Loaded Inverted Pendulum Model of Legged Locomotion Using Surface Interpolation.
%16th Internation Conference on Dynamical Systems, Theory and Aplications
%(DSTA 2021), December 6-9, 2021, On-line.
%doi.org/10.34658/9788366741201
%
%SLIP_stab_bound    plots a chosen parameterset as a point onto the 3D and/or 
%2D boundary of the stable parameter region for the spring loaded inverted
%pendulum model.
%
%   SLIP_stab_bound(gamma, beta, Ehat)
%   Plots the 2D stability boundary curve at the given gamma with the custom 
%   gamma (dimensionless stiffness), beta (leg angle) and Ehat (dimensionless 
%   mechanical energy) parameterset denoted by a +.
%   If at the custom gamma the boundary is not calculated or it does not 
%   exist, a warning will appear.
%
%   SLIP_stab_bound(gamma, beta, Ehat, dim)
%   dim specifies the dimensions of the stability diagram. If not specified, 
%   2D diagram will be plotted.
%       0   -   2D and 3D plots
%       2   -   2D plot [x,y] axis are [beta,Ehat], black curve is the
%       stability boundary, blue is the mu=0 eigenvalue curve
%       3   -   3D plot [x,y,z] axis are [gamma,beta,Ehat], black surface is the
%       stability boundary, blue is the mu=0 eigenvalue surface
%
%   SLIP_stab_bound(gamma, beta, Ehat, dim, N)
%   N specifies the resolution of the plot. The resolution if not specified is 100.
%
%   SLIP_stab_bound(gamma, beta, Ehat,..., fit1,...., fit6)
%   Shows up to six fitted parameter curves on the 3D figure:
%       'human' - Humans from [1]; color: teal blue (#0048BA)
%       'rhea' - Rheas from [1]; color: amber (#FF7E00)
%       'kangaroo' - Kangarooos from [1]; color: opera mauve (#B284BE)
%       'dog' - Dogs from [1]; color: electric green (#00CC00)
%       'running' - Different running speeds from [2]; color: mustard (#FFE14D)
%       'stridefreq' - Different stride frequencies from [3]; color: crimson (#AF002A)
%
%   Try this setting for a 3D plot with resulotion 50x50:
%   SLIP_stab_bound(50., 0.5, 2., 3, 50)
%   Try this setting for a 3D plot with default resulotion, humans data are presented:
%   SLIP_stab_bound(50., 0.5, 2., 3, 'human')
%
%[1] Cavagna, G.A., Franzetti, P., Heglund, N.C., Willems, P.: The determinants of the step frequency in running, trotting and hopping in man and other vertebrates. The Journal of Physiology 399, 8192 (1988). Doi:10.3389/fbioe.2018.00141
%[2] Hobara, H., Inoue, K., Gomi, K., Sakamoto, M., Muraoka, T., Iso, S., Kanosue, K.: Continuous change in spring-mass characteristics during a 400m sprint. Journal of Science and Medicine in Sport 13(2), 13491353 (2010). Doi:10.1016/j.jsams.2009.02.002
%[3] Morin, J.B., Samozino, P., Zameziati, K., Belli, A.: Effects of altered stride frequency and contact time on leg-spring behavior in human running. Journal of Biomechanics 40(15), 33413348 (2007). Doi:10.1016/j.jbiomech.2007.05.001


% Assign input, detect problems with input
if nargin > 11
    error('Too many input agrument.')
elseif nargin < 3
    error('Not enough input agrument.')
end

args = varargin;

%default dim and N, if specified, it is overwritten
dim = 2;
N = 100;

%assigning inputs
inum = 0;
itxt = 0;
txtmax = zeros(6,1);
for i = 1:nargin
    txt = ischar(args{i});
    if txt == 0
        if inum > 4
            error('Too many input agrument (numbers).')
        end
        inum = inum+1;
        switch inum
            case 1
                gamma = args{i};
            case 2
                beta = args{i};
            case 3
                Ehat = args{i};
            case 4
                dim = args{i};
            case 5
                N = args{i};
        end
    else
        if itxt > 5
            error('Too many input agrument (string).')
        end 
        itxt = itxt+1;
        txtmax(itxt) = i;
    end
end

if inum < 3
    error('Please choose values for all three parameters: gamma, beta, Ehat')
else
    if dim == 0 || dim == 2 || dim == 3
    else
        error('dim must be either 0, 2 or 3')
    end
    if dim == 2 && itxt > 0
        warning('Fitted parameters will not be presented on the 2D figure, please choose 0 or 3 as input for dim.')
    end
end

mfileloc = which('SLIP_stab_bound');
[filepath,~,~] = fileparts(mfileloc);
Gamma_load=load([filepath,'/Stab_Bound_Data/gammas_surf_vegpont']);
Gammas_sort = sort(Gamma_load.gammas);

% Detect problem of gamma input
if gamma > Gammas_sort(end)
    error(['Stability boundary is not calculated here, please choose a value ',num2str(Gammas_sort(end)),' or lower for gamma'])
elseif gamma < Gammas_sort(1)
    if dim == 0
        warning(['There is no 2D stability region at gamma values lower than ',num2str(Gammas_sort(1)),', therefore only 3D plot will be presented'])
        dim = 3;
    elseif dim == 2
        error(['There is no 2D stability region at gamma values lower than ',num2str(Gammas_sort(1))])
    end
end

s = linspace(0,1,N);

if dim == 0
	% plot the stability region
    plotting_2D_figures(gamma,beta,Ehat,s,Gammas_sort)        
	plotting_3D_figures(gamma,beta,Ehat,s)
        
	% plot the fitted parameters
	plotting_fitted_parameters(args,txtmax,itxt,filepath)
    
elseif dim == 2
	% plot the stability region
    plotting_2D_figures(gamma,beta,Ehat,s,Gammas_sort)
    
elseif dim == 3        
	% plot the stability region
    plotting_3D_figures(gamma,beta,Ehat,s)
    
    % plot the fitted parameters
	plotting_fitted_parameters(args,txtmax,itxt,filepath)
end

%%

function [Gamma,Beta,Ehat,Gamma_0,Beta_0,Ehat_0] = surf_stab_bound(X,Y)
Beta = zeros(size(Y));
Ehat = zeros(size(Y));

if length(X)== 1
    X = X*ones(size(Y,1),1);
end

Gamma_load=load('Stab_Bound_Data/gammas_surf_vegpont');

Gammas_sort = sort(Gamma_load.gammas);

Gammas_norm = (Gammas_sort-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));

Gamma = Gammas_sort(1)+X*(Gammas_sort(end)-Gammas_sort(1));
for i = 1:size(X,2)
    flag_veg = 0;
    k = 2;
    while X(1,i) > Gammas_norm(k)
        k = k+1;
    end
    
    if k == 2
        A_m=load('Stab_Bound_Data/spline_veg');
        flag_veg = 1;
    else
        A_m=load(['Stab_Bound_Data/spline_gamma_',num2str(floor(Gammas_sort(k-1))),'_',num2str(100*(Gammas_sort(k-1)-floor(Gammas_sort(k-1))))]);
        x1 = Gammas_norm(k-1);
    end
    A_p=load(['Stab_Bound_Data/spline_gamma_',num2str(floor(Gammas_sort(k))),'_',num2str(100*(Gammas_sort(k)-floor(Gammas_sort(k))))]);
    x2 = Gammas_norm(k);
    
    %     [x1,x2]
    for j = 1:size(Y,1)
        k = 1;
        while Y(j,i) > A_m.sp_betas(k,4)
            k = k+1;
        end
        fb1 = polyval(A_m.sp_betas(k,1:3),Y(j,i));
        fE1 = polyval(A_m.sp_Ebars(k,1:3),Y(j,i));
        if flag_veg == 1
            fg1 = polyval(A_m.sp_gammas(k,1:3),Y(j,i));
            x1 = (fg1-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));
        end
        
        if x1>X(1,i)
            Gamma(j,i)=nan;
            Beta(j,i)=nan;
            Ehat(j,i)=nan;
        else
            k = 1;
            while Y(j,i) > A_p.sp_betas(k,4)
                k = k+1;
            end
            fb2 = polyval(A_p.sp_betas(k,1:3),Y(j,i));
            fE2 = polyval(A_p.sp_Ebars(k,1:3),Y(j,i));
            
            
            df1 = (fb2-fb1)/(x2-x1);
            Beta_eh = [(-fb1 + fb2 + df1.*(x1 - x2))./(x1 - x2).^2;
                (2*fb1.*x1 - 2*fb2.*x1 + df1.*(-x1.^2 + x2.^2))./(x1 - x2).^2;
                (fb2.*x1.^2 + df1.*x1.*(x1 - x2).*x2 + fb1.*x2.*(-2*x1 + x2))./(x1 - x2).^2]';
            
            df1 = (fE2-fE1)/(x2-x1);
            E_eh = [(-fE1 + fE2 + df1.*(x1 - x2))./(x1 - x2).^2;
                (2*fE1.*x1 - 2*fE2.*x1 + df1.*(-x1.^2 + x2.^2))./(x1 - x2).^2;
                (fE2.*x1.^2 + df1.*x1.*(x1 - x2).*x2 + fE1.*x2.*(-2*x1 + x2))./(x1 - x2).^2]';
            
            Beta(j,i) = polyval(Beta_eh,X(1,i));
            Ehat(j,i) = polyval(E_eh,X(1,i));
        end
    end
end

[Gamma_0,Beta_0,Ehat_0] = surf_0(X,Y);

function [Gamma,Beta,Ehat] = surf_0(X,Y)
Beta = zeros(size(Y));
Ehat = zeros(size(Y));

Gamma_load=load('Stab_Bound_Data/gammas_surf_vegpont');

Gammas_sort = sort(Gamma_load.gammas);

Gammas_norm = (Gammas_sort-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));

Gamma = Gammas_sort(1)+X*(Gammas_sort(end)-Gammas_sort(1));
bad_index = 0;
for i = 1:size(X,2)
    k = 2;
    while X(1,i) > Gammas_norm(k)
        k = k+1;
    end
    
    if floor(Gammas_sort(k-1)) > 9
        
        A_m=load(['Stab_Bound_Data/spline_gamma_',num2str(floor(Gammas_sort(k-1))),'_',num2str(100*(Gammas_sort(k-1)-floor(Gammas_sort(k-1)))),'_0']);
        x1 = Gammas_norm(k-1);
        A_p=load(['Stab_Bound_Data/spline_gamma_',num2str(floor(Gammas_sort(k))),'_',num2str(100*(Gammas_sort(k)-floor(Gammas_sort(k)))),'_0']);
        x2 = Gammas_norm(k);
        
        %     [x1,x2]
        for j = 1:size(Y,1)
            k = 1;
            while Y(j,i) > A_m.sp_betas(k,4)
                k = k+1;
            end
            fb1 = polyval(A_m.sp_betas(k,1:3),Y(j,i));
            fE1 = polyval(A_m.sp_Ebars(k,1:3),Y(j,i));
            
            k = 1;
            while Y(j,i) > A_p.sp_betas(k,4)
                k = k+1;
            end
            fb2 = polyval(A_p.sp_betas(k,1:3),Y(j,i));
            fE2 = polyval(A_p.sp_Ebars(k,1:3),Y(j,i));
            
            
            df1 = (fb2-fb1)/(x2-x1);
            Beta_eh = [(-fb1 + fb2 + df1.*(x1 - x2))./(x1 - x2).^2;
                (2*fb1.*x1 - 2*fb2.*x1 + df1.*(-x1.^2 + x2.^2))./(x1 - x2).^2;
                (fb2.*x1.^2 + df1.*x1.*(x1 - x2).*x2 + fb1.*x2.*(-2*x1 + x2))./(x1 - x2).^2]';
            
            df1 = (fE2-fE1)/(x2-x1);
            E_eh = [(-fE1 + fE2 + df1.*(x1 - x2))./(x1 - x2).^2;
                (2*fE1.*x1 - 2*fE2.*x1 + df1.*(-x1.^2 + x2.^2))./(x1 - x2).^2;
                (fE2.*x1.^2 + df1.*x1.*(x1 - x2).*x2 + fE1.*x2.*(-2*x1 + x2))./(x1 - x2).^2]';
            
            Beta(j,i) = polyval(Beta_eh,X(1,i));
            Ehat(j,i) = polyval(E_eh,X(1,i));
        end
    else
        bad_index = i;
    end
end

Gamma = Gamma(:,bad_index+1:end);
Beta = Beta(:,bad_index+1:end);
Ehat = Ehat(:,bad_index+1:end);

function plotting_2D_figures(gamma,beta,Ehat,s,Gammas_sort)

gamma_n = (gamma-Gammas_sort(1))/(Gammas_sort(end)-Gammas_sort(1));
[~,beta_bound,ehat_bound,~,beta_0,ehat_0] = surf_stab_bound(gamma_n,s');
        
figure()
hold on
grid on
plot(beta_bound,ehat_bound,'k')
plot(beta_0,ehat_0,'b')
plot(beta,Ehat,'r+','linewidth',2,'markersize',12)
xlabel('beta [1]')
ylabel('Ehat [1]')

function plotting_3D_figures(gamma,beta,Ehat,s)

[X,Y] = meshgrid(s);
[Gamma_bound,Beta_bound,Ehat_bound,Gamma_0,Beta_0,Ehat_0] = surf_stab_bound(X,Y);
figure()
hold on
grid on
surf_bound = surf(Gamma_bound,Beta_bound,Ehat_bound,'FaceAlpha',0.5);
set(surf_bound, 'edgecolor', 'none', 'facecolor', [0 0 0], 'FaceLighting', 'gouraud')
surf_0 = surf(Gamma_0,Beta_0,Ehat_0,'FaceAlpha',0.5);
set(surf_0, 'edgecolor', 'none', 'facecolor', [0.3010 0.7450 0.9330], 'FaceLighting', 'gouraud')
plot3(gamma,beta,Ehat,'r+','linewidth',2,'markersize',12)
plot3([gamma,gamma],[beta,beta],[0,Ehat],'r')
view([1,1,1])
xlabel('gamma [1]')
ylabel('beta [1]')
zlabel('Ehat [1]')

function plotting_fitted_parameters(args,txtmax,itxt,filepath)

for i = 1:itxt
    fitname=args{txtmax(i)};
    plot_exist = 1;
    switch fitname
        case 'human' 
            Fitpar = load([filepath,'/Stab_Bound_Data/CavagnaHuman']);
            ColorDef = [0.00,0.28,0.73];
        case 'rhea' 
            Fitpar = load([filepath,'/Stab_Bound_Data/CavagnaRhea']);
            ColorDef = [1.00,0.49,0.00];
        case 'kangaroo'
            Fitpar = load([filepath,'/Stab_Bound_Data/CavagnaKangaroo']);
            ColorDef = [0.70,0.52,0.75];
        case 'dog'
            Fitpar = load([filepath,'/Stab_Bound_Data/CavagnaDog5']);
            ColorDef = [0.00,0.80,0.00];
        case 'running'
            Fitpar = load([filepath,'/Stab_Bound_Data/HobaraTable1']);
            ColorDef = [1.00,0.88,0.30];
        case 'stridefreq'
            Fitpar = load([filepath,'/Stab_Bound_Data/MorinFigure34']);
            ColorDef = [0.69,0.00,0.16];
        otherwise
        warning(['Unrecognized variable: ', fitname])
        plot_exist = 0;
    end
    if plot_exist == 1
        plot3(Fitpar.gammas,Fitpar.betas,Fitpar.Ebars, 's-', 'color', ColorDef)
    end
end


